@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/hash.c,v 1.3 2003/07/25 23:47:54 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/hash.c,v 1.7 2004/04/16 05:26:45 keithp Exp $ */
 
 /*
  * Copyright © 2003 Keith Packard and Bart Massey.
@@ -216,6 +216,14 @@ HashPrint (Value f, Value av, char format, int base, int width, int prec, int fi
     }
     if (pretty)
     {
+	if (ht->def)
+	{
+	    if (!first)
+		FilePuts (f, ",");
+	    FilePuts (f, " => ");
+	    Print (f, ht->def, format, base, width, prec, fill);
+	    first = False;
+	}
 	if (!first)
 	    FilePuts (f, " ");
 	FilePuts (f, "}");
@@ -249,6 +257,7 @@ HashMark (void *object)
     MemReference (ht->type);
     MemReference (ht->keyType);
     MemReference (ht->elts);
+    MemReference (ht->def);
 }
 
 ValueRep    HashRep = { 
@@ -286,6 +295,7 @@ NewHash (Bool constant, TypePtr keyType, TypePtr type)
     ret->hash.type = type;
     ret->hash.keyType = keyType;
     ret->hash.elts = 0;
+    ret->hash.def = 0;
     Resize (&ret->hash, &hashSets[0]);
     RETURN (ret);
 }
@@ -300,9 +310,21 @@ HashGet (Value hv, Value key)
     he = Find (ht, hash, key);
     if (!HashEltValid (he))
     {
-	RaiseStandardException (exception_uninitialized_value,
-				"uninitialized hash element", 0);
-	return (Void);
+	if (!ht->def)
+	{
+	    RaiseStandardException (exception_uninitialized_value,
+				    "uninitialized hash element", 0);
+	    return (Void);
+	}
+	if (ht->count >= ht->hashSet->entries && 
+	    ht->hashSet != &hashSets[NHASHSETS - 1])
+	{
+	    Resize (ht, ht->hashSet + 1);
+	}
+	ht->count++;
+	HashEltHash(he) = hash;
+	HashEltKey(he) = key;
+	HashEltValue(he) = Copy(ht->def);
     }
     return HashEltValue (he);
 }
@@ -314,7 +336,7 @@ HashSet (Value hv, Value key, Value value)
     Value	    hash = ValueHash (key);
     Value	    *he;
 
-    if (ht->count == ht->hashSet->entries && 
+    if (ht->count >= ht->hashSet->entries && 
 	ht->hashSet != &hashSets[NHASHSETS - 1])
     {
 	Resize (ht, ht->hashSet + 1);
@@ -327,6 +349,14 @@ HashSet (Value hv, Value key, Value value)
     HashEltValue (he) = value;
 }
 
+void
+HashSetDef (Value hv, Value def)
+{
+    HashTablePtr    ht = &hv->hash;
+
+    ht->def = Copy(def);
+}
+
 Value
 HashRef (Value hv, Value key)
 {
@@ -346,6 +376,8 @@ HashRef (Value hv, Value key)
 	ht->count++;
 	HashEltHash (he) = hash;
 	HashEltKey (he) = key;
+	if (ht->def)
+	    HashEltValue (he) = Copy(ht->def);
     }
     RETURN (NewRef (ht->elts, 
 		    &HashEltValue(he) - BoxElements (ht->elts)));
@@ -395,7 +427,7 @@ HashKeys (Value hv)
     {
 	if (HashEltValid (e)) 
 	{
-	    BoxValueSet (keys->array.values, i, HashEltKey (e));
+	    ArrayValueSet (&keys->array, i, HashEltKey (e));
 	    i++;
 	}
 	HashEltStep (e);
@@ -410,6 +442,7 @@ HashCopy (Value hv)
 {
     ENTER ();
     Value   new = NewHash (False, hv->hash.keyType, hv->hash.type);
+    new->hash.def = hv->hash.def;
     Resize (&new->hash, hv->hash.hashSet);
     Rehash (hv->hash.elts, &new->hash);
     RETURN (new);
