@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/main.c,v 1.35 2004/02/27 03:50:16 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/main.c,v 1.37 2004/04/16 05:26:45 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -31,10 +31,10 @@ setArgv (int argc, char **argv)
 
     args = NewArray (True, True, typePrim[rep_string], 1, &argc);
     for (i = 0; i < argc; i++)
-	BoxValueSet (args->array.values, i, NewStrString (argv[i]));
+	ArrayValueSet (&args->array, i, NewStrString (argv[i]));
     setVar (GlobalNamespace, "argv", args, 
 	    NewTypeArray (typePrim[rep_string],
-			   NewExprTree (COMMA, 0, 0)));
+			   NewExprTree (COMMA, 0, 0), False));
     EXIT ();
 }
 
