@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/natural.c,v 1.23 2004/02/27 03:50:16 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/natural.c,v 1.24 2004/05/27 23:30:55 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -666,7 +666,7 @@ NaturalCompliment (Natural *a, int len)
 
     resultlen = length (a);
     at = data(a) + (resultlen-1);
-    while (resultlen > 0 && ~*at == 0)
+    while (resultlen > len && ~*at == 0)
     {
 	resultlen--;
 	at--;
