@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/sched.c,v 1.54 2004/02/27 03:50:16 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/sched.c,v 1.55 2004/04/16 05:26:45 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -1069,11 +1069,11 @@ RaiseException (Value thread, SymbolPtr except, Value args, InstPtr *next)
 	PrintError ("Unhandled exception %A (", except->symbol.name);
 	if (args)
 	{
-	    BoxPtr  values = args->array.values;
-	    for (i = 0; i < values->nvalues; i++)
+	    int	    dim = ArrayLimits(&args->array)[0];
+	    for (i = 0; i < dim; i++)
 	    {
-		PrintError ("%v", BoxValueGet (values, i));
-		if (i < values->nvalues - 1)
+		PrintError ("%v", ArrayValueGet (&args->array, i));
+		if (i < dim - 1)
 		    PrintError (", ");
 	    }
 	}
@@ -1135,10 +1135,10 @@ RaiseStandardException (StandardException   se,
     va_start (va, argc);
     i = argc + 1;
     args = NewArray (False, False, typePoly, 1, &i);
-    BoxValueSet (args->array.values, 0, NewStrString (string));
+    ArrayValueSet (&args->array, 0, NewStrString (string));
     if (argc)
 	for (i = 0; i < argc; i++)
-	    BoxValueSet (args->array.values, i + 1, va_arg (va, Value));
+	    ArrayValueSet (&args->array, i + 1, va_arg (va, Value));
     standardException = se;
     standardExceptionArgs = args;
     SetSignalException ();
