@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/scope.c,v 1.30 2004/04/16 21:41:52 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/scope.c,v 1.31 2004/06/17 20:30:47 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -226,6 +226,7 @@ NamespaceLocate (Value		names,
     int		    i;
     NamespacePtr    namespace;
     FramePtr	    f;
+    CodePtr	    c;
     Value	    string;
     NamelistPtr	    namelist = 0;
     SymbolPtr	    symbol;
@@ -240,7 +241,7 @@ NamespaceLocate (Value		names,
 				NewInt (0), names);
 	return False;
     }
-    GetNamespace (&namespace, &f);
+    GetNamespace (&namespace, &f, &c);
     for (i = 0; i < ArrayLimits(&names->array)[0]; i++)
     {
 	string = ArrayValue (&names->array, i);
