@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/string.c,v 1.17 2004/04/16 21:41:52 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/string.c,v 1.18 2004/06/08 09:30:54 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -255,3 +255,15 @@ NewStrString (char *str)
     strcpy (StringChars (&ret->string), str);
     RETURN (ret);
 }
+
+Value
+NewCharString (int c)
+{
+    ENTER ();
+    int	    size = StringCharSize (c);
+    Value   ret = NewString (size);
+
+    StringPutChar (c, StringChars (&ret->string));
+    RETURN (ret);
+}
+
