@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/value.c,v 1.46 2004/04/16 05:26:45 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/value.c,v 1.47 2004/05/28 01:26:38 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -462,7 +462,7 @@ ShiftR (Value av, Value bv)
 	Sign	sign = Positive;
 	if (Negativep (av))
 	{
-	    av = Minus (av, ShiftL (One, Minus (bv, One)));
+	    av = Minus (av, Minus (ShiftL (One, bv), One));
 	    sign = Negative;
 	}
 	av = Reduce (NewInteger (sign,
