@@ -1,4 +1,4 @@
-/* $Header: /local/src/CVS/nickle/value.h,v 1.104 2004/05/28 02:58:14 keithp Exp $ */
+/* $Header: /local/src/CVS/nickle/value.h,v 1.106 2004/06/14 06:43:33 keithp Exp $ */
 
 /*
  * Copyright © 1988-2004 Keith Packard and Bart Massey.
@@ -471,8 +471,6 @@ typedef struct _argList {
 } ArgList;
 
 extern Type	    *typePoly;
-extern Type	    *typeGroup;
-extern Type	    *typeField;
 extern Type	    *typeRefPoly;
 extern Type	    *typeFileError;
 extern Type	    *typeArrayInt;
@@ -508,6 +506,8 @@ Bool	TypeCompatibleAssign (Type *dest, Value v);
 Bool	TypeIsSupertype (Type *super, Type *sub);
 /* a is a supertype of b or b is a supertype of a */
 Bool	TypeIsOrdered (Type *a, Type *b);
+/* a and b are 'cotypes' */
+Bool	TypeIsCotype (Type *a, Type *b);
 
 #define TypePoly(t)	((t)->base.tag == type_prim && (t)->prim.prim == rep_undef)
 #define TypeBool(t)	((t)->base.tag == type_prim && (t)->prim.prim == rep_bool)
@@ -961,6 +961,7 @@ extern DataCachePtr	refCache;
 
 Value	NewString (long length);
 Value	NewStrString (char *);
+Value	NewCharString (int c);
 Value	NewArray (Bool constant, Bool resizable, TypePtr type, int ndim, int *dims);
 void	ArrayResize (Value av, int dim, int size);
 void	ArraySetDimensions (Value av, int *dims);
